@echo OFF
@if not "%ECHO%"=="" echo %ECHO%
@if not "%OS%"=="Windows_NT" goto DOSEXIT
rem $Workfile: makeusr.bat $ $Revision: 1 $ $Date: 12/04/97 9:51a $
rem $Archive: /TimH/Pubs/Books/Macmillan/Windows NT Scripting/Scripts/makeusr.bat $

rem Set local scope and call MAIN procedure
setlocal & pushd & set RET=
	set SCRIPTNAME=%~n0
	set SCRIPTPATH=%~f0
	if "%DEBUG%"=="1" (set TRACE=echo) else (set TRACE=rem)
	call _mtplib :INIT %SCRIPTPATH%
	if /i {%1}=={/help} (call :HELP %2) & (goto :HELPEXIT)
	if /i {%1}=={/?} (call :HELP %2) & (goto :HELPEXIT)
	call :MAIN %*
	:HELPEXIT
popd & endlocal & set RET=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem HELP procedure
rem Display brief on-line help message
rem
:HELP
if defined TRACE %TRACE% [proc %0 %*]
	echo Syntax: MAKEUSR username [password ^| *] [switches]
	echo Create a new user NT user account.
	echo .
	echo /DOMAIN
	echo     Create account in domain (def: local computer).
	echo /LOCAL:group
	echo     Add account to specified local group. Multiple /LOCAL
	echo     switches are allowed. In addition, a list of groups
	echo     may be supplied in the MULOCAL variable.
	echo /GLOBAL:group
	echo     Add account to specified global group. Multiple /GLOBAL
	echo     switches are allowed. In addition, a list of groups
	echo     may be supplied in the MUGLOBAL variable.
	echo /FULLNAME:name
	echo     Specify full name for account. Use quotes if the name
	echo     contains spaces.
	echo /PROFILEPATH:path
	echo     Profile path for this account.
	echo /SCRIPTPATH:path
	echo     Logon script path for this account.
	echo /HOMEDIR:path
	echo     User home directory. If /HOMECOMPUTER is NOT specified,
	echo     this is a local path. If /HOMECOMPUTER is specified, this is
	echo     a local path RELATIVE TO THAT COMPUTER (e.g. C: refers to
	echo     C: on that computer).
	echo /HOMECOMPUTER:name
	echo     Computer to contain home directory (def: local computer).
	echo /HOMEPROTO:uncpath
	echo     UNC name of prototype home directory. Copied to home
	echo     directory when created.
	echo /COMMENT:text
	echo     Text comment for this user name.
	echo MUNETUSER
	echo     Use this variable to specify additional switches to the
	echo     NET USER command. Useful when adding lots of similar
	echo     accounts in a batch.
	echo MUXCACLS
	echo     Use this variable to specify additional switches to the
	echo     XCACLS command.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAIN procedure
rem
:MAIN
if defined TRACE %TRACE% [proc %0 %*]
	rem If no arguments, we default to displaying help
	if {%1}=={} (call :HELP) & (goto :EOF)

	rem Parse command line and setup variables
	set CMDLINE=%*
	call _mtplib :PARSECMDLINE 0
	if %CMDARGCOUNT% LSS 2 (call :HELP) & (goto :EOF)
	call _mtplib :GETARG 1
	set USERNAME=%RET%
	call _mtplib :GETARG 2
	set PASSWORD=%RET%

	rem Process command line switches
	set DOMAINSW=
	set HOMEDIR=
	set HOMECOMPUTER=
	set HOMEPROTO=
	set /a IX=1
	:GETSWITCHLOOP
		call _mtplib :GETSWITCH %IX%
		if "%RET%"=="" goto :GETSWITCHLOOPEND
		set /a IX+=1
		if /i "%RET%"=="/domain"  set DOMAINSW=/DOMAIN
		if /i "%RET%"=="/homedir" set HOMEDIR=%RETV%
		if /i "%RET%"=="/homecomputer" set HOMECOMPUTER=%RETV%
		if /i "%RET%"=="/homeproto" set HOMEPROTO=%RETV%
		if /i "%RET%"=="/fullname" set MUNETUSER=%MUNETUSER% /FULLNAME:"%RETV%"
		if /i "%RET%"=="/profilepath" set MUNETUSER=%MUNETUSER% /PROFILEPATH:"%RETV%"
		if /i "%RET%"=="/scriptpath" set MUNETUSER=%MUNETUSER% /SCRIPTPATH:"%RETV%"
		if /i "%RET%"=="/comment" set MUNETUSER=%MUNETUSER% /COMMENT:"%RETV%"
		if /i "%RET%"=="/local" set MULOCAL=%MULOCAL% "%RETV%"
		if /i "%RET%"=="/global" set MUGLOBAL=%MUGLOBAL% "%RETV%"
	goto :GETSWITCHLOOP
	:GETSWITCHLOOPEND

	rem Get domain name
	set DOMAINNAME=
	if not "%DOMAINSW%"=="" for /f "delims=\" %%I in ('whoami') do set DOMAINNAME=%%I\
	
	rem Proceed to create the account
	call :MAKEHOMEDIR "%HOMEDIR%"
	call :CREATEUSER
	call :SECUREHOMEDIR
	call :ADDLOCALGROUPS %MULOCAL%
	call :ADDGLOBALGROUPS %MUGLOBAL%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAKEHOMEDIR procedure
rem Create the home directory/share for the account
rem
rem Arguments:	HOMEDIR=home directory (local or remote)
rem		HOMECOMPUTER=computer name (empty for local)
rem
:MAKEHOMEDIR
if defined TRACE %TRACE% [proc %0 %*]
	if "%HOMEDIR%"=="" goto :EOF
	set HOMEDRIVE=%~d1%
	set HOMEPATH=%~pnx1
	set HOMEADMIN=\\%HOMECOMPUTER%\%HOMEDRIVE:~0,1%$%HOMEPATH%
	if "%HOMECOMPUTER%"=="" (
		(md "%HOMEDIR%")
	) else (
		(md "%HOMEADMIN%")
		(echo --- rmtshare \\%HOMECOMPUTER%\%USERNAME%$="%HOMEDIR%" /remark:"%USERNAME% home directory")
		(rmtshare \\%HOMECOMPUTER%\%USERNAME%$="%HOMEDIR%" /remark:"%USERNAME% home directory")
		(set HOMEDIR=\\%HOMECOMPUTER%\%USERNAME%$)
	)
	if not "%HOMEPROTO%"=="" (
		(echo --- xcopy "%HOMEPROTO%" "%HOMEDIR%" /e/i/q/r/k)
		(xcopy "%HOMEPROTO%" "%HOMEDIR%" /e/i/q/r/k)
	)
	sleep 2
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem CREATEUSER procedure
rem Execute the NET USER command to create the new account
rem
rem Arguments:	USERNAME=user name
rem		PASSWORD=new password or * for interactive
rem		MUNETUSER=switches for NET USER
rem		DOMAINSW=/DOMAIN switch (if present)
rem
:CREATEUSER
if defined TRACE %TRACE% [proc %0 %*]
	if "%HOMEDIR%"=="" (set HOMEDIRSW=) else (set HOMEDIRSW=/HOMEDIR:"%HOMEDIR%")
	echo --- net user %USERNAME% %PASSWORD% /add %MUNETUSER% %DOMAINSW% %HOMEDIRSW%
	net user %USERNAME% %PASSWORD% /ADD %MUNETUSER% %DOMAINSW% %HOMEDIRSW%
	sleep 2
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SECUREHOMEDIR procedure
rem Secure the home directory for the user
rem
rem Arguments:	HOMEDIR=path for home directory (may be UNC)
rem
:SECUREHOMEDIR
if defined TRACE %TRACE% [proc %0 %*]
	if "%HOMECOMPUTER%"=="" (
		(echo --- xcacls %HOMEDIR% /t /g %DOMAINNAME%%USERNAME%:f;f %MUXCACLS% 2^>nul)
		(xcacls %HOMEDIR% /t /g %DOMAINNAME%%USERNAME%:f;f %MUXCACLS% 2^>nul)
	) else (
		(echo --- xcacls %HOMEADMIN% /t /g %DOMAINNAME%%USERNAME%:f;f %MUXCACLS% 2^>nul)
		(xcacls %HOMEADMIN% /t /g %DOMAINNAME%%USERNAME%:f;f %MUXCACLS% 2^>nul)
	)
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem ADDLOCALGROUPS procedure
rem Add user to each specified local group
rem
rem Arguments:	%n=names of local groups
rem
:ADDLOCALGROUPS
if defined TRACE %TRACE% [proc %0 %*]
	if {%1}=={} goto :EOF
	echo --- net localgroup %1 %USERNAME% /add %DOMAINSW%
	net localgroup %1 %USERNAME% /add %DOMAINSW%
	shift /1
	goto :ADDLOCALGROUPS
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem ADDGLOBALGROUPS procedure
rem Add user to each specified global group
rem
rem Arguments:	%n=names of global groups
rem
:ADDGLOBALGROUPS
if defined TRACE %TRACE% [proc %0 %*]
	if {%1}=={} goto :EOF
	echo --- net group %1 %USERNAME% /add %DOMAINSW%
	net group %1 %USERNAME% /add %DOMAINSW%
	shift /1
	goto :ADDGLOBALGROUPS
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem Additional procedures go here...

rem These must be the FINAL LINES in the script...
:DOSEXIT
echo This script requires Windows NT

rem /////////////////////////////////////////////////////////////////////////
