@echo OFF
@if not "%ECHO%"=="" echo %ECHO%
@if not "%OS%"=="Windows_NT" goto DOSEXIT
rem $Workfile: usrquota.bat $ $Revision: 1 $ $Date: 12/04/97 9:51a $
rem $Archive: /TimH/Pubs/Books/Macmillan/Windows NT Scripting/Scripts/usrquota.bat $

rem Set local scope and call MAIN procedure
setlocal & pushd & set RET=
	set SCRIPTNAME=%~n0
	set SCRIPTPATH=%~f0
	if "%DEBUG%"=="1" (set TRACE=echo) else (set TRACE=rem)
	call _mtplib :INIT %SCRIPTPATH%
	if /i {%1}=={/help} (call :HELP %2) & (goto :HELPEXIT)
	if /i {%1}=={/?} (call :HELP %2) & (goto :HELPEXIT)
	call :MAIN %*
	:HELPEXIT
popd & endlocal & set RET=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem HELP procedure
rem Display brief on-line help message
rem
:HELP
if defined TRACE %TRACE% [proc %0 %*]
	echo Syntax: USRQUOTA dir-list [/QUOTA] [/DOMAIN] [/OVER]
	echo Check disk quota usage per user.
	echo .
	echo dir-list
	echo     One or more directories to check (UNC names are ok).
	echo /QUOTA
	echo     Only report on users with restricted quotas.
	echo /DOMAIN
	echo     Gets quota info from domain (def: local computer).
	echo /OVER
	echo     Only display users over quota limit.
	echo .
	echo To specify a quota for a user account, add the text #QUOTA:nnn#
	echo to the comment (description) field of the user account, where
	echo nnn in the user quota in megabytes.
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAIN procedure
rem
:MAIN
if defined TRACE %TRACE% [proc %0 %*]
	rem If no arguments, we default to displaying help
	if {%1}=={} (call :HELP) & (goto :EOF)

	rem Parse command line and setup variables
	set CMDLINE=%*
	call _mtplib :PARSECMDLINE 0
	if %CMDARGCOUNT% LSS 1 (call :HELP) & (goto :EOF)
	call _mtplib :FINDSWITCH /quota
	if not "%RET%"=="0" (set QUOTASW=1) else (set QUOTASW=0)
	call _mtplib :FINDSWITCH /domain
	if not "%RET%"=="0" (set DOMAINSW=/DOMAIN) else (set DOMAINSW=)
	call _mtplib :FINDSWITCH /over
	if not "%RET%"=="0" (set OVERSW=1) else (set OVERSW=0)

	rem Get temporary filenames
	call _mtplib :GETTEMPNAME
	set TEMPALL=%RET%
	call _mtplib :GETTEMPNAME
	set TEMPONE=%RET%

	rem Accumulate disk usage results into temporary file
	set /a IX=1
	:DISKUSELOOP
		call _mtplib :GETARG %IX%
		if "%RET%"=="" goto :DISKUSELOOPEND
		set /a IX+=1
		if exist "%TEMPONE%" del "%TEMPONE%"
		diskuse "%RET%" /T /S /F:%TEMPONE% >nul
		type %TEMPONE% | find "," >>"%TEMPALL%"
	goto :DISKUSELOOP
	:DISKUSELOOPEND

	rem Now parse the results file
	set USERNAME=
	echo Username                       Used     Quota    Percent
	echo ------------------------------ -------- -------- --------
	for /f "tokens=1-3 delims=," %%I in ('type %TEMPALL% ^| sort') do call :ADDQUOTA %%I\%%J %%K
	call :SHOWUSERINFO

	rem Delete temporary files
	del %TEMPALL%
	del %TEMPONE%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem ADDQUOTA procedure
rem Add a line of quota information
rem
rem Arguments:	%1=user name
rem		%2=quota bytes
rem
:ADDQUOTA
	rem Handle switch to new username
	if not {%1}=={%USERNAME%} (
		(call :SHOWUSERINFO)
		(set USERNAME=%1)
		(set /a MBTOTAL=0)
	)
	set /a MBTOTAL+=%2/1048576
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem SHOWUSERINFO procedure
rem Display accumulated results for one user
rem
rem Arguments:	USERNAME=user name
rem		MBTOTAL=total mbytes for this user
rem
:SHOWUSERINFO
if defined TRACE %TRACE% [proc %0 %*]
	if "%USERNAME%"=="" goto :EOF

	rem Get quota information for user
	set MBQUOTA=
	set /a PERCENT=0
	for /f "tokens=2 delims=\" %%I in ("%USERNAME%") do set USER=%%I
	for /f "tokens=*" %%I in ('net user %USER% %DOMAINSW% 2^>nul ^| find /i "#QUOTA:"') do set MBQUOTA=%%I
	for /f "tokens=2 delims=#" %%I in ("%MBQUOTA%") do set MBQUOTA=%%I
	for /f "tokens=2 delims=:" %%I in ("%MBQUOTA%") do set MBQUOTA=%%I
	if not "%MBQUOTA%"=="" set /a PERCENT=MBTOTAL*100/MBQUOTA

	rem Filter display based upon switches
	set T5=
	if "%MBQUOTA%"=="" if "%QUOTASW%"=="1" goto :EOF
	if not "%MBQUOTA%"=="" if %PERCENT% LEQ 100 if "%OVERSW%"=="1" goto :EOF
	if not "%MBQUOTA%"=="" if %PERCENT% GTR 100 set T5=***
	if "%MBQUOTA%"=="" set PERCENT=---

	rem The following SET commands are all space-padded to here ------->
	set T1=%USERNAME%                                                   
	set T2=%MBTOTAL%                                                   
	set T3=%MBQUOTA%                                                   
	set T4=%PERCENT%%%                                                   
	
	rem Do not pad these SET commands                                                  
	set T1=%T1:~0,30%
	set T2=%T2:~0,8%
	set T3=%T3:~0,8%
	set T4=%T4:~0,8%
	echo %T1% %T2% %T3% %T4% %T5%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem Additional procedures go here...

rem These must be the FINAL LINES in the script...
:DOSEXIT
echo This script requires Windows NT

rem /////////////////////////////////////////////////////////////////////////
