@echo OFF
@if not "%ECHO%"=="" echo %ECHO%
@if not "%OS%"=="Windows_NT" goto DOSEXIT
rem $Workfile: xcptest.bat $ $Revision: 1 $ $Date: 12/04/97 9:51a $
rem $Archive: /TimH/Pubs/Books/Macmillan/Windows NT Scripting/Scripts/xcptest.bat $

rem Set local scope and call MAIN procedure
setlocal & pushd & set RET=
	set SCRIPTNAME=%~n0
	set SCRIPTPATH=%~f0
	if "%DEBUG%"=="1" (set TRACE=echo) else (set TRACE=rem)
	call _mtplib :INIT %SCRIPTPATH%
	if /i {%1}=={/help} (call :HELP %2) & (goto :HELPEXIT)
	if /i {%1}=={/?} (call :HELP %2) & (goto :HELPEXIT)
	call :MAIN %*
	:HELPEXIT
popd & endlocal & set RET=%RET%
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem HELP procedure
rem Display brief on-line help message
rem
:HELP
if defined TRACE %TRACE% [proc %0 %*]
	echo Syntax: XCPTEST ref-dir test-dir [...] /PASSES:n /CHECK:m
	echo Ping-pong XCOPY machine soak test.
	echo .
	echo Copies a reference directory in round-robin fashion among
	echo two or more test dirs, checking for file corruption or loss.
	echo .
	echo The source dir is ref-dir. If two or more test-dirs are
	echo specified, these are used round-robin. If one test-dir is
	echo specified, two sub-dirs (test1 and test2) are used in this
	echo test-dir. ALL FILES IN THE TEST DIRECTORIES ARE DELETED!!!
	echo .
	echo Use /PASSES to specify the number of passes (def: infinite).
	echo Use /CHECK to specify pass counts between each file check.
	echo (def: 10). Use /NOSTOP to prevent test stopping on error(s).
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem MAIN procedure
rem
:MAIN
if defined TRACE %TRACE% [proc %0 %*]
	rem If no arguments, we default to displaying help
	if {%1}=={} (call :HELP) & (goto :EOF)

	rem Parse command line and setup variables
	set CMDLINE=%*
	call _mtplib :PARSECMDLINE 0
	if %CMDARGCOUNT% LSS 2 (call :HELP) & (goto :EOF)
	call _mtplib :FINDSWITCH /passes
	if "%RET%"=="0" (set /a PASSES=0) else (set /a PASSES=RETV)
	call _mtplib :FINDSWITCH /check
	if "%RET%"=="0" (set /a CHECKEVERY=10) else (set /a CHECKEVERY=RETV)
	call _mtplib :FINDSWITCH /nostop
	if "%RET%"=="0" (set /a NOSTOP=0) else (set /a NOSTOP=1)
	call _mtplib :FINDSWITCH /y
	if "%RET%"=="0" (set NOCONFIRM=0) else (set NOCONFIRM=1)
	call _mtplib :GETARG 1
	set REFDIR="%RET%"
	if %CMDARGCOUNT% GTR 2 goto :USERLIST
		call _mtplib :GETARG 2
		set DIRLIST="%RET%\test1" "%RET%\test2"
	goto :DIRLISTDONE
	:USERLIST
		set DIRLIST=
		set ARG=2
		:USERLISTLOOP
			if %ARG% GTR %CMDARGCOUNT% goto :USERLISTLOOPEND
			call _mtplib :GETARG %ARG%
			set DIRLIST=%DIRLIST% "%RET%"
			set /a ARG+=1
		goto :USERLISTLOOP
		:USERLISTLOOPEND
	:DIRLISTDONE

	rem Verify everything is ok before starting
	if not "%NOCONFIRM%"=="1" (
		echo All files in these directories will be deleted:
		echo     %DIRLIST%
		%COMSPEC% /c choice /n "Delete all files?"
		if errorlevel 2 goto :EOF
	)

	rem Copy reference directory
	call :DELETETESTDIRS %DIRLIST%
	call :COPYREFDIR %REFDIR% %DIRLIST%

	rem Now process each test pass
	set /a PASS=0
	set /a ERRORS=0
	set /a CHECK=CHECKEVERY
	:MAINPASSLOOP
		if %PASSES% NEQ 0 if %PASS% GEQ %PASSES% goto :MAINPASSLOOPEND
		set /a PASS+=1
		echo XCPTEST: Pass %PASS% of %PASSES%, %ERRORS% errors
		call :COPYDIRLIST %DIRLIST%
		set /a CHECK-=1
		if %CHECK% GTR 0 goto :SKIPCHECK
			call :CHECKDIR %REFDIR% %DIRLIST%
			set /a CHECK=CHECKEVERY
		:SKIPCHECK
		if %NOSTOP% EQU 0 if %ERRORS% GTR 0 goto :MAINPASSLOOPEND
		%COMSPEC% /c choice /n /T:Y,5 "Continue testing?"
		if errorlevel 2 goto :MAINPASSLOOPEND
	goto :MAINPASSLOOP
	:MAINPASSLOOPEND
	if %ERRORS% EQU 0 call :CHECKDIR %REFDIR% %DIRLIST%
	if %ERRORS% EQU 0 call :DELETETESTDIRS %DIRLIST%
	echo XCPTEST: End of test, %PASS% passes, %ERRORS% errors
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem CHECKDIR procedure
rem Check test dir against refdir
rem
rem Arguments:	%1=ref directory
rem		%2=test directory
rem
:CHECKDIR
	set D1=%1
	set D2=%2
	set D1=%D1:"=%
	set D2=%D2:"=%
	for /f "tokens=*" %%I in ('fc /b %D1%\*.* %D2%\*.* ^| find /v "Comparing" ^| find /v "FC: no"') do (set /a ERRORS+=1) & (echo %%I)
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem DELETETESTDIRS procedure
rem Delete test directories
rem
rem Arguments:	%n=list of all directories
rem
:DELETETESTDIRS
	echo Deleteing directory %1...
	if {%1}=={} goto :EOF
	if exist %1 del %1 /f/q
	if not exist %1 md %1
	shift /1
	goto :DELETETESTDIRS
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem COPYREFDIR procedure
rem Copy reference directory to 1st test directory
rem
rem Arguments:	%1=refdir
rem		%2=first test dir
rem
:COPYREFDIR
	echo Copying reference directory...
	xcopy %1 %2 /r
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem COPYDIRLIST procedure
rem Copy directories in round-robin list
rem
rem Arguments:	%n=list of all directories
rem
:COPYDIRLIST
	if {%2}=={} goto :EOF
	set FIRST=%1
	:COPYDIRLISTLOOP
		if {%2}=={} goto :COPYDIRLISTLOOPEND
		xcopy %1 %2 /r/q
		shift /1
	goto :COPYDIRLISTLOOP
	:COPYDIRLISTLOOPEND
	xcopy %1 %FIRST% /r/q
goto :EOF

rem /////////////////////////////////////////////////////////////////////////
rem Additional procedures go here...

rem These must be the FINAL LINES in the script...
:DOSEXIT
echo This script requires Windows NT

rem /////////////////////////////////////////////////////////////////////////
